@@ -2,7 +2,7 @@ import atexit
 import os
 import webbrowser
 from websocket import create_connection
-
+import traceback
 from .exit_hook import hooks
 from .utils import serialize, deserialize
 from .parameters import set_params
@@ -11,6 +11,7 @@ import os
 initialized = False
 
 
+# TODO: create_connection should have retry logic
 def initialize():
     global ws, initialized
     initialized = True
@@ -38,13 +39,29 @@ def initialize():
 def send(data):
     if not initialized:
         return
-    ws.send(serialize(data))
+    debug = {
+        "debug": {
+            "stack": [
+                {
+                    "filename": summary.filename,
+                    "lineno": summary.lineno,
+                    "name": summary.name,
+                }
+                for summary in traceback.extract_stack()
+            ]
+        }
+    }
+    ws.send(serialize({**data, **debug}))
 
 
 def receive(path: str = ""):
     if not initialized:
         return
-    data = deserialize(ws.recv())
+
+    recvd = ws.recv()
+    if not recvd:
+        raise Exception("Websocket not connected")
+    data = deserialize(recvd)
 
     if not path:
         return data
@@ -53,7 +70,7 @@ def receive(path: str = ""):
 
 @atexit.register
 def close():
-    if not initialized:
+    if not initialized or ws is None or not ws.connected:
         return
     send(
         {
